@@ -1,104 +1,202 @@
-from __future__ import annotations
-
 import asyncio
 import logging
 import threading
+from abc import ABC, abstractmethod
 from concurrent.futures import ThreadPoolExecutor
+from typing import Any
 
-# import functools
-from typing import Any, Awaitable
-from xml.dom import NotFoundErr
-
+from pynoticenter.noticenter_observer import PyNotiObserver, PyNotiObserverCollection
 from pynoticenter.options import PyNotiOptions
 from pynoticenter.task import PyNotiTask
 from pynoticenter.task_queue import PyNotiTaskQueue
 
 
-class PyNotiObserver(object):
-    __fn: callable = None
-    __options: PyNotiOptions = None
+class PyNotiCenterInterface(ABC):
+    """PyNotiCenter Interface"""
 
-    def __init__(self, fn: callable, options: PyNotiOptions):
-        self.__fn = fn
-        self.__options = options
+    @abstractmethod
+    def post_task(self, fn: callable, *args: Any, **kwargs: Any) -> str:
+        """post task to default task queue.
 
-    @property
-    def fn(self) -> callable:
-        return self.__fn
+        Args:
+            fn (callable): callback function
+            *args (Any): args
+            **kwargs (Any): kwargs
 
-    @property
-    def options(self) -> PyNotiOptions:
-        return self.__options
+        Returns:
+            str: return task id
+        """
+        pass
 
+    @abstractmethod
+    def post_task_with_delay(self, delay: float, fn: callable, *args: Any, **kwargs: Any) -> str:
+        """post task to default task queue with delay.
 
-class PyNotiObserverCollection:
-    __lock: threading.RLock = None
-    __name: str = ""
-    __fn_list: list[PyNotiObserver] = None
-    __receiver_observers_dict: dict[Any, list[PyNotiObserver]] = None
-    __scheduler: callable = None
+        Args:
+            fn (callable): callback function
+            delay (float): delay time in seconds.
+            *args (Any): args
+            **kwargs (Any): kwargs
 
-    def __init__(self, name: str, scheduler: callable):
-        self.__scheduler = scheduler
-        self.__lock = threading.RLock()
-        self.__name = name
-        self.__fn_list = list[PyNotiObserver]()
-        self.__receiver_observers_dict = dict[Any, list[PyNotiObserver]]()
+        Returns:
+            str: return task id
+        """
+        pass
 
-    def add_observer(self, fn: callable, receiver: Any | None = None, *, options: PyNotiOptions | None = None):
-        if fn is None:
-            return
+    @abstractmethod
+    def post_task_to_task_queue(self, queue_name: str, fn: callable, *args: Any, **kwargs: Any) -> str:
+        """post task to named task queue.
 
-        with self.__lock:
-            if receiver is None:
-                self.__fn_list.append(PyNotiObserver(fn, options))
-                return
+        Args:
+            fn (callable): callback function
+            queue_name (str): queue name, create from create_task_queue
+            *args (Any): args
+            **kwargs (Any): kwargs
 
-            if receiver in self.__receiver_observers_dict:
-                self.__receiver_observers_dict[receiver].append(PyNotiObserver(fn, options))
-            else:
-                self.__receiver_observers_dict[receiver] = list([PyNotiObserver(fn, options)])
+        Returns:
+            str: return task id
+        """
+        pass
 
-    def remove_observer(self, fn: callable, receiver: Any | None = None):
-        def remove_fn(item: PyNotiObserver) -> bool:
-            return item.fn == fn
+    @abstractmethod
+    def cancel_task(self, task_id):
+        """cancel task from default task queue with task id
 
-        with self.__lock:
-            if receiver is None:
-                self.__fn_list = list(filter(remove_fn, self.__fn_list))
-                return
+        Args:
+            task_id (str): task id
 
-            if receiver not in self.__receiver_observers_dict:
-                return
+        """
+        pass
 
-            observers = self.__receiver_observers_dict.pop(receiver)
-            observers = list(filter(remove_fn, observers))
-            if len(observers) > 0:
-                self.__receiver_observers_dict[receiver] = observers
+    @abstractmethod
+    def cancel_task_with_queue_name(self, queue_name: str, task_id: str):
+        """cancel task from named task queue with task id
 
+        Args:
+            queue_name (str): task queue name
+            task_id (str): task id
+
+        """
+        pass
+
+    @abstractmethod
+    def wait_until_task_complete(self):
+        """wait until all task complete. it will block until there is no task."""
+        pass
+
+    @abstractmethod
+    def shutdown(self, wait: bool):
+        """shutdown PyNotiCenter. Once shutdown, you can't call it anymore.
+
+        When shutdown is called, you can not post new task to task queue, all task queue will ignore the new task.
+        When wait is setted, it will block until all task complete, if not set, all task queue will shutdown as fast as possible.
+
+        Args:
+            wait (bool): set wait for task complete or not.
+
+        """
+        pass
+
+    @abstractmethod
+    def release_task_queue(self, queue_name: str, wait: bool):
+        """release task queue
+
+        Args:
+            queue_name (str): task queue name
+            wait (bool): wait for wait complete.
+
+        """
+        pass
+
+    @abstractmethod
+    def create_task_queue(self, queue_name: str) -> PyNotiTaskQueue:
+        """create task queue with name
+
+        Args:
+            queue_name (str): task queue name
+
+        Returns:
+            PyNotiTaskQueue: task queue
+
+        """
+        pass
+
+    @abstractmethod
+    def get_default_task_queue(self) -> PyNotiTaskQueue:
+        """return default task queue
+
+        Returns:
+            PyNotiTaskQueue: default task queue
+        """
+        pass
+
+    @abstractmethod
+    def get_task_queue(self, queue_name: str) -> PyNotiTaskQueue:
+        """return named task queue
+
+        Args:
+            queue_name (str): task queue name
+
+        Returns:
+            PyNotiTaskQueue: task queue
+        """
+        pass
+
+    @abstractmethod
+    def add_observer(self, name: str, fn: callable, receiver: Any = None, *, options: PyNotiOptions = None):
+        """Add observer to PyNotiCenter
+
+        Args:
+            name (str): notification name
+            fn (callable): callback function
+            receiver (Any): receiver object
+            options (PyNotiOptions): options
+        """
+        pass
+
+    @abstractmethod
+    def remove_observer(self, name: str, fn: callable, receiver: Any = None):
+        """Remove observer from PyNotiCenter
+
+        Args:
+            name (str): notification name
+            fn (callable): callback function
+            receiver (Any): receiver object
+        """
+        pass
+
+    @abstractmethod
     def remove_observers(self, receiver: Any):
-        if receiver is None:
-            return
-        with self.__lock:
-            if receiver in self.__receiver_observers_dict:
-                self.__receiver_observers_dict.pop(receiver)
+        """Remove observers from PyNotiCenter
 
+        Args:
+            receiver (Any): receiver object
+        """
+        pass
+
+    @abstractmethod
     def remove_all_observers(self):
-        with self.__lock:
-            self.__fn_list.clear()
-            self.__receiver_observers_dict.clear()
+        """Remove all observers from PyNotiCenter"""
+        pass
 
-    def notify_observers(self, *args: Any, **kwargs: Any):
-        observers = list[PyNotiObserver]()
-        with self.__lock:
-            observers.extend(self.__fn_list)
-            for _, v in self.__receiver_observers_dict.items():
-                observers.extend(v)
-        for observer in observers:
-            self.__scheduler(observer, *args, **kwargs)
+    @abstractmethod
+    def notify_observers(self, name: str, *args: Any, **kwargs: Any):
+        """Notify observers
+
+        Args:
+            name (str): notification name
+            *args (Any): args
+            **kwargs (Any): kwargs
+        """
+        pass
 
 
-class PyNotiCenter:
+class PyNotiCenter(PyNotiCenterInterface):
+    """PyNotiCenter Implement
+
+    :meta private:
+    """
+
     global __default_global_instance
     __default_global_instance = None
     global __default_global_lock
@@ -127,7 +225,7 @@ class PyNotiCenter:
         self.__scheduler_thread.start()
 
     @staticmethod
-    def default() -> PyNotiCenter:
+    def default() -> PyNotiCenterInterface:
         global __default_global_lock
         global __default_global_instance
         with __default_global_lock:
@@ -136,11 +234,9 @@ class PyNotiCenter:
         return __default_global_instance
 
     def post_task(self, fn: callable, *args: Any, **kwargs: Any) -> str:
-        """Post task to default task queue."""
         return self.post_task_with_delay(0, fn, *args, **kwargs)
 
     def post_task_with_delay(self, delay: float, fn: callable, *args: Any, **kwargs: Any) -> str:
-        """Post task with delay to default task queue."""
         with self.__lock:
             return self.__default_queue.post_task_with_delay(delay, fn, *args, **kwargs)
 
@@ -185,11 +281,6 @@ class PyNotiCenter:
                 event.wait(timeout=0.5)
 
     def shutdown(self, wait: bool):
-        """Shutdown all tasks, include the unnamed task queue.
-
-        Args:
-            wait (bool): wait until all task done.
-        """
         logging.info(f"PyNotiCenter start shutdown, wait = {wait}")
         task_queues = list[PyNotiTaskQueue]()
         with self.__lock:
@@ -214,12 +305,6 @@ class PyNotiCenter:
         logging.info("PyNotiCenter shutdown end")
 
     def release_task_queue(self, queue_name: str, wait: bool):
-        """release task queue resource.
-
-        Args:
-            queue_name (str): queue name
-            wait (bool): wait until task done
-        """
         if queue_name is None:
             return
         with self.__lock:
@@ -228,17 +313,6 @@ class PyNotiCenter:
                 queue.terminate(wait)
 
     def create_task_queue(self, queue_name: str) -> PyNotiTaskQueue:
-        """Create task queue by name.
-
-        If name always exist, it will return the existen queue.
-        If name is None, it will create unnamed task queue.
-
-        Args:
-            queue_name (str): queue name
-
-        Returns:
-            PyNotiTaskQueue: task queue
-        """
         with self.__lock:
             if self.__is_shutdown:
                 logging.error(f"fail on create task queue {queue_name}. PyNotiCenter is shutdown.")
@@ -265,17 +339,6 @@ class PyNotiCenter:
             return self.__default_queue
 
     def get_task_queue(self, queue_name: str) -> PyNotiTaskQueue:
-        """Get task queue from notification center.
-
-        If name is None, return default task queue.
-
-        Args:
-            queue_name (str): queue name
-
-        Returns:
-            PyNotiTaskQueue: return task queue
-        """
-
         if queue_name is None:
             return self.__default_queue
 
@@ -297,10 +360,7 @@ class PyNotiCenter:
             logging.info("scheduler thread end.")
             self.__scheduler_thread = None
 
-    def add_observer(
-        self, name: str, fn: callable, receiver: Any | None = None, *, options: PyNotiOptions | None = None
-    ):
-        """add notification observer"""
+    def add_observer(self, name: str, fn: callable, receiver: Any = None, *, options: PyNotiOptions = None):
         with self.__lock:
             observer_collection: PyNotiObserverCollection = None
             if name in self.__notifications_dict:
@@ -312,7 +372,7 @@ class PyNotiCenter:
             observer_collection.add_observer(fn, receiver, options=options)
             self.__notifications_dict[name] = observer_collection
 
-    def remove_observer(self, name: str, fn: callable, receiver: Any | None = None):
+    def remove_observer(self, name: str, fn: callable, receiver: Any = None):
         observer_collection = self.__get_notification_observer_collection__(name)
         if observer_collection is not None:
             observer_collection.remove_observer(fn, receiver)
@@ -329,7 +389,6 @@ class PyNotiCenter:
             self.__notifications_dict.clear()
 
     def notify_observers(self, name: str, *args: Any, **kwargs: Any):
-        """post notification"""
         observer_collection = self.__get_notification_observer_collection__(name)
         if observer_collection is not None:
             observer_collection.notify_observers(*args, **kwargs)
